using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.IO;

namespace myseq  {
	public class MapPoint  {
		public int x = 0;
		public int y = 0;
		public int z = 0;
	}		
	public class MapLine  {
		public MapLine()  {
		}

		
		public string name = "";
		public Pen color = null;
		public uint numpoints = 0;
		public MapPoint []points;

	}

	public class MapText  {
		public string text = "";
		public SolidBrush color = null;
		public int x = 0;
		public int y = 0;
	}


	/// <summary>
	/// Summary description for MapCon.
	/// </summary>
	public class MapCon : System.Windows.Forms.UserControl {
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		//~~
		// Hand relocation variables
		private Cursor hCurs = null;
		private int hx, hy;
		private int ox, oy;
		private bool hDown;

		public Font drawFont = new Font("Arial", 10);
		public bool abc = false;

		public ArrayList mobs = new ArrayList();
		public uint nummobs = 0;
		SolidBrush []conColors = new SolidBrush [256];

		//public const int MAX_LINES	= 8192;
		public string longname = "";
		public string shortname = "";
		public int minx = 0;
		public int maxx = 0;
		public int miny = 0;
		public int maxy = 0;
		public int minz = 0;
		public int maxz = 0;
		public ArrayList lines = new ArrayList();//MapLine[MAX_LINES];
		public ArrayList texts = new ArrayList();//MapText[50];
		public int numtexts = 0;
		public int numlines = 0;
		public float scale = 1.0f;
		public int offsetx = 0;
		public int offsety = 0;
		public int offsetxg = 0;
		public int offsetyg = 0;
		public int mapx = 0;
		public int mapy = 0;
		public float rawscale = 0.0f;
	
		public int playerx = 0;
		public int playery = 0;
		public float playerhead = 0;

		public int selectedID = -1;
		public SPAWNINFO playerinfo = new SPAWNINFO();
		public string curTarget = "";
		public Bitmap offScreenBmp = null;
		public Graphics g = null;

		public MapCon() {
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			offScreenBmp = new Bitmap(this.Width, this.Height); 
			g = Graphics.FromImage(offScreenBmp);

			SetStyle(ControlStyles.AllPaintingInWmPaint|ControlStyles.UserPaint|ControlStyles.Opaque, true);
		}


		public void onResize()  {
			/*MapPane mp = (MapPane)Parent;
			Form1 f1 = (Form1)mp.Parent;*/

			if (Width > 0 && Height > 0)  {//(f1.WindowState != FormWindowState.Minimized)  {
				offScreenBmp = new Bitmap(this.Width, this.Height); 
				g = Graphics.FromImage(offScreenBmp);
			}
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing ) {
			if( disposing ) {
				if(components != null) {
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent() {
			// 
			// MapCon
			// 
			this.AutoScroll = true;
			this.Name = "MapCon";
			this.Size = new System.Drawing.Size(432, 360);
			this.Load += new System.EventHandler(this.MapCon_Load);
			this.Paint += new System.Windows.Forms.PaintEventHandler(this.Form1_Paint);
			this.MouseDown += new System.Windows.Forms.MouseEventHandler(this.MapCon_Click);
			this.MouseWheel += new MouseEventHandler(this.MapCon_MouseScroll);
			this.KeyPress += new KeyPressEventHandler(this.MapCon_KeyPress);
			this.MouseDown += new MouseEventHandler(this.MapCon_MouseDown);
			this.MouseUp += new MouseEventHandler(this.MapCon_MouseUp);
			this.MouseMove += new MouseEventHandler(this.MapCon_MouseMove);

			//~~
			// Load the Hand Cursor
			hCurs = new Cursor(".\\Hand.cur");

			// Initialize hx,hy to 0,0 and set semiphore false
			hx = hy = 0;
			hDown = false;
		}
		#endregion


		public bool loadMap(string filename)  {
			StreamReader tr;
			string line = "";
			numtexts = 0;
			
			lines.Clear();
			texts.Clear();

			try  {
				tr = new StreamReader(File.OpenRead(filename));
			}
			catch (System.IO.FileNotFoundException)  {
				string str = "File not found: ";
				str += filename;

				MessageBox.Show(str, "Convert Error");
				return false;
			}

			// First line is file ID info
			line = tr.ReadLine();	

			if (line == null)
				return false;
	
			string tok = "";

			if ((longname = getnexttoken(ref line, ',')) == null)
				return false;				

			if ((shortname = getnexttoken(ref line, ',')) == null)
				return false;				
			
			// ICK....
			if ((tok = getnexttoken(ref line, ',')) != null)  {
				minx = int.Parse(tok);			
				if ((tok = getnexttoken(ref line, ',')) != null)  {
					miny = int.Parse(tok);
					if ((tok = getnexttoken(ref line, ',')) != null)  {
						minz = int.Parse(tok);
						if ((tok = getnexttoken(ref line, ',')) != null)  {
							maxx = int.Parse(tok);			
							if ((tok = getnexttoken(ref line, ',')) == null)  {
								maxy = int.Parse(tok);
								if ((tok = getnexttoken(ref line, ',')) == null)   {
									maxz = int.Parse(tok);
								}
							}
						}
					}
				}
			}
			
			// Now the bulk of the file....
			// Note A and Z are not implemented asthey don't ever seem to be used...
			numlines = 0;
			while ((line = tr.ReadLine()) != null)  {
				if (line != "")  {
					if ((tok = getnexttoken(ref line, ',')) == null)
						return false;				
					
					if (tok == "Z")  {
						// Ignore the ZEM one
					}
					else if (tok == "M")  {
						MapLine work = new MapLine();

						work = new MapLine();
						if ((tok = getnexttoken(ref line, ',')) == null)
							return false;				
						work.name = tok;

						if ((tok = getnexttoken(ref line, ',')) == null)
							return false;				
						work.color = new Pen(new SolidBrush(stringToColor(tok)));					

						if ((tok = getnexttoken(ref line, ',')) == null)
							return false;		
						work.numpoints = uint.Parse(tok);
						work.points = new MapPoint[work.numpoints];
					
						for (int c = 0; c < work.numpoints; c++)  {
							work.points[c] = new MapPoint();
							if ((tok = getnexttoken(ref line, ',')) != null)	 {
								work.points[c].x = (int)-float.Parse(tok);
								if ((tok = getnexttoken(ref line, ',')) != null)  {
									work.points[c].y = (int)-float.Parse(tok);
									if ((tok = getnexttoken(ref line, ',')) != null)  {
										work.points[c].z = (int)float.Parse(tok);
									}
									else  {
										work.numpoints--;
										break;
									}
								}
								else  {
									work.numpoints--;
									break;
								}
							}
							else  {
								work.numpoints--;
								break;
							}
						}
						lines.Add(work);
						numlines++;
					}
					else if (tok == "L")  {
						MapLine work = new MapLine();
						if ((work.name = getnexttoken(ref line, ',')) == null)
							return false;				

						if ((tok = getnexttoken(ref line, ',')) == null)
							return false;				
						work.color = new Pen(new SolidBrush(stringToColor(tok)));					
						
						if ((tok = getnexttoken(ref line, ',')) == null)
							return false;		
						work.numpoints = uint.Parse(tok);
						work.points = new MapPoint[work.numpoints];
					
						for (int c = 0; c < work.numpoints; c++)  {
							work.points[c] = new MapPoint();							
							if ((tok = getnexttoken(ref line, ',')) != null)	 {
								work.points[c].x = (int)-float.Parse(tok);
								if ((tok = getnexttoken(ref line, ',')) != null)  {
									work.points[c].y = (int)-float.Parse(tok);
								}
								else  {
									work.numpoints--;
									break;
								}
							}
							else  {
								work.numpoints--;
								break;
							}						
						}
						lines.Add(work);
						numlines++;
					}
					else if (tok == "P")  {
						MapText work = new MapText();
						if ((work.text = getnexttoken(ref line, ',')) == null)
							return false;				
						
						if ((tok = getnexttoken(ref line, ',')) == null)
							return false;		
						work.color = new SolidBrush(stringToColor(tok));;

						if ((tok = getnexttoken(ref line, ',')) == null)
							return false;		
						work.x = -int.Parse(tok);

						if ((tok = getnexttoken(ref line, ',')) == null)
							return false;		
						work.y = -int.Parse(tok);

						texts.Add(work);
						numtexts++;
					}
				}

			}

			tr.Close();

			updateExtents();

			Invalidate();

			return true;
		}

		public bool loadLoYMap(string filename)  {
			StreamReader tr;
			string line = "";
			numtexts = 0;
			string tok = "";
			
			lines.Clear();
			texts.Clear();

			try  {
				tr = new StreamReader(File.OpenRead(filename));
			}
			catch (System.IO.FileNotFoundException)  {
				string str = "File not found: ";
				str += filename;

				MessageBox.Show(str, "Convert Error");
				return false;
			}

			numlines = 0;
			while ((line = tr.ReadLine()) != null)  {
				if (line != "")  {
					if ((tok = getnexttoken(ref line, ' ')) == null)
						return false;				
					
					if (tok == "L")  {
						MapLine work = new MapLine();
						work.points = new MapPoint[2];
						work.numpoints = 2;

						work.points[0] = new MapPoint();
						work.points[1] = new MapPoint();

						if ((tok = getnexttoken(ref line, ',')) == null)
							return false;		
						work.points[0].x = (int)float.Parse(tok);
						
						if ((tok = getnexttoken(ref line, ',')) == null)
							return false;		
						work.points[0].y = (int)float.Parse(tok);

						if ((tok = getnexttoken(ref line, ',')) == null)
							return false;		
						work.points[0].z = (int)float.Parse(tok);

						if ((tok = getnexttoken(ref line, ',')) == null)
							return false;		
						work.points[1].x = (int)float.Parse(tok);
						
						if ((tok = getnexttoken(ref line, ',')) == null)
							return false;		
						work.points[1].y = (int)float.Parse(tok);

						if ((tok = getnexttoken(ref line, ',')) == null)
							return false;		
						work.points[1].z = (int)float.Parse(tok);

						int r = 0, g = 0, b = 0;
						if ((tok = getnexttoken(ref line, ',')) == null)
							return false;		
						r = int.Parse(tok);
						
						if ((tok = getnexttoken(ref line, ',')) == null)
							return false;		
						g = int.Parse(tok);

						if ((tok = getnexttoken(ref line, ',')) == null)
							return false;		
						b = int.Parse(tok);

						work.color = new Pen(new SolidBrush(Color.FromArgb(r, g, b)));
						lines.Add(work);
						numlines++;
					}
					else if (tok == "P")  {
						MapText work = new MapText();

						if ((tok = getnexttoken(ref line, ',')) == null)
							return false;		
						work.x = (int)float.Parse(tok);

						if ((tok = getnexttoken(ref line, ',')) == null)
							return false;		
						work.y = (int)float.Parse(tok);

						if ((tok = getnexttoken(ref line, ',')) == null)
							return false;		

						int r = 0, g = 0, b = 0;
						if ((tok = getnexttoken(ref line, ',')) == null)
							return false;		
						r = int.Parse(tok);
						
						if ((tok = getnexttoken(ref line, ',')) == null)
							return false;		
						g = int.Parse(tok);

						if ((tok = getnexttoken(ref line, ',')) == null)
							return false;		
						b = int.Parse(tok);

						work.color = new SolidBrush(Color.FromArgb(r, g, b));

						if ((tok = getnexttoken(ref line, ',')) == null)
							return false;		
						// Dunno what this one does...

						if ((tok = getnexttoken(ref line, ',')) == null)
							return false;		
						work.text = tok;
						texts.Add(work);
						numtexts++;
					}
				}
			}

			longname = filename;	
			shortname = filename;
			tr.Close();
			
			updateExtents();

			Invalidate();

			return true;

		}

		private void MapCon_Load(object sender, System.EventArgs e) {
		}

		private void MapCon_MouseScroll(object sender, MouseEventArgs me)  {
			MapPane mp = (MapPane)Parent;
			Form1 f1 = (Form1)mp.Parent;

			if (scale+(float)(me.Delta)/600.0f >= 0.1)  {
				scale = scale+(float)(me.Delta)/600.0f;		
				f1.mapPane1.scale.Value = (decimal)(scale*100);
			}

			Invalidate();
		}

		private void MapCon_KeyPress(object sender, KeyPressEventArgs e)  {
			MapPane mp = (MapPane)Parent;
			Form1 f1 = (Form1)mp.Parent;

			if (e.KeyChar == '+')  {
				scale = scale+0.2f;
				f1.mapPane1.scale.Value = (decimal)(scale*100);
				Invalidate();
			}
			else if (e.KeyChar == '-')  {
				if (scale-0.2 >= 0.1)  {
					scale = scale-0.2f;
					f1.mapPane1.scale.Value = (decimal)(scale*100);
				}
				Invalidate();
			}
		}

		private void MapCon_Click(object sender, MouseEventArgs me)  {
			int ox = (int)((Width/2 - (me.X-offsetxg))/(rawscale*scale));
			int oy = (int)((Height/2 - (me.Y-offsetyg))/(rawscale*scale));

			int x = (int)(playerinfo.Y)+ox;
			int y = (int)(playerinfo.X)+oy;

			MapPane mp = (MapPane)Parent;
			Form1 f1 = (Form1)mp.Parent;

			lock(mobs)  {
				foreach(SPAWNINFO sp in mobs)  {
					if (sp.Y < x+(5/(rawscale*scale))&& sp.Y > x-(5/(rawscale*scale)) && sp.X < y+(5/(rawscale*scale)) && sp.X > y-(5/(rawscale*scale)))  {
						selectedID = (int)sp.SpawnID;

						foreach(ListViewItem it in f1.listBox1.Items)  {
							if (it.Text == sp.Name)  {
								it.Selected = true;
								it.EnsureVisible();
							}
						}
						break;
					}
				}
			}
		}

		//~~
		private void MapCon_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			System.Windows.Forms.Cursor.Current = hCurs;

			// Remember the mouse down loc and set semiphore
			hx = e.X;
			hy = e.Y;
			hDown = true;

			//MapPane mp = (MapPane)Parent;
			//ox = mp.offsetx.Value;
			//oy = mp.offsety.Value;

			ox = offsetxg;
			oy = offsetyg;
		}

		//~~
		private void MapCon_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			hDown = false;
			hx = hy = 0;

			//offsetx = offsetxg;
			//offsety = offsetyg;

			System.Windows.Forms.Cursor.Current = System.Windows.Forms.Cursors.Default;		
		}

		//~~
		private void MapCon_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			int dx, dy;

			// If we are dragging, then change the origin.
			if (hDown) 
			{
				// Compute delta x,y from original click
				dx = hx - e.X;
				dy = hy - e.Y;

				offsetxg = (ox + dx);
				offsetyg = (oy + dy);

				MapPane mp = (MapPane)Parent;
				mp.offsetx.Value = offsetxg;
				mp.offsety.Value = offsetyg;

				Invalidate();
			}
		}
		public string getnexttoken(ref string s, char seperator)  
		{
			int c = 0;
			string token = "";

			if (s.Length == 0)
				return null;

			while (c < s.Length && s[c] != seperator)  {
				token += s[c];
				c++;
			}
			c++;

			if (c == s.Length+1)
				s = "";
			else
				s = s.Substring(c, s.Length-c);
			return token;
		}

		public Color stringToColor(string s)  {
			s = s.ToLower();
			if (s == "gray")
				return Color.LightGray;
			else if (s == "blue")
				return Color.Blue;
			else if (s == "darkred")
				return Color.DarkRed;
			else if (s == "red")
				return Color.Red;
			else if (s == "magenta")
				return Color.Magenta;
			else if (s == "cyan")
				return Color.Cyan;
			else if (s == "darkblue")
				return Color.DarkBlue;
			else if (s == "darkcyan")
				return Color.DarkCyan;
			else if (s == "yellow")
				return Color.Yellow;
			else if (s == "white")
				return Color.White;
			else if (s == "black")
				return Color.Black;
			else if (s == "darkgreen")
				return Color.DarkGreen;
			else if (s == "green")
				return Color.Green;
			else if (s == "peru")
				return Color.Peru;
			else if (s == "yellow3")
				return Color.Yellow;
			else if (s == "skyblue")
				return Color.SkyBlue;
			else if (s == "brown")
				return Color.Brown;
			else if (s == "mediumorchid")
				return Color.MediumOrchid;
			else if (s == "tan")
				return Color.Tan;
			else if (s == "orange")
				return Color.Orange;
			else if (s == "purple")
				return Color.Purple;

			return Color.Lime;

		}


		public void updateExtents()  {
			if (this.longname != "")  {
				minx = ((MapLine)(lines[0])).points[0].x;
				maxx = ((MapLine)(lines[0])).points[0].x;
				miny = ((MapLine)(lines[0])).points[0].y;
				maxy = ((MapLine)(lines[0])).points[0].y;
				minz = ((MapLine)(lines[0])).points[0].z;
				maxz = ((MapLine)(lines[0])).points[0].z;

				int c, d;

				//for(d = 0; d < numlines; d++)  {
				foreach (MapLine l in lines)  {
					for (c = 0; c < l.numpoints; c++)  {
						if (l.points[c].x > maxx)
							maxx = l.points[c].x;
						else if (l.points[c].x < minx)
							minx = l.points[c].x;

						if (l.points[c].y > maxy)
							maxy = l.points[c].y;
						else if (l.points[c].y < miny)
							miny = l.points[c].y;

						if (l.points[c].z > maxz)
							maxz = l.points[c].z;
						else if (l.points[c].z < minz)
							minz = l.points[c].z;
					}
				}

				// Work out the 100% scale
				float tmpscale = ((float)(Width)/((float)(maxx-minx)));

				if ((float)(Height)/((float)(maxy-miny)) < tmpscale)
					tmpscale = ((float)(Height)/((float)(maxy-miny)));
				//scale = tmpscale;
				rawscale = tmpscale;//-0.005f;

				int midx = (int)((maxx+minx)/2);
				int midy = (int)((maxy+miny)/2);

				offsetx = (int)((Width/2)-(midx*(rawscale*scale)));
				offsety = (int)((Height/2)-(midy*(rawscale*scale)));

				MapPane mp = (MapPane)Parent;
				Form1 f1 = (Form1)mp.Parent;

				f1.mapPane1.scale.Value = 100;
			}
		}

		public void fillConColors()  {
			int level = 0;
			if (Settings.Instance.LevelOverride == -1)
				level = playerinfo.Level;
			else 
				level = Settings.Instance.LevelOverride;

			int c = 0;
			for (c = 0; c < 256; c++)  
				conColors[c] = new SolidBrush(Color.Red);

			int greenRange = 0; 
			int cyanRange = 0; 

			if (level < 8) { // 1 - 7 
				greenRange = -4;
				cyanRange = -8;
			} 
			else if (level < 13) { // 8 - 12 
				greenRange = -5;
				cyanRange = -4;
			}
			else if (level < 23) { // 
				greenRange = -7;
				cyanRange = -5;
			}
			else if (level < 27) { //
				greenRange = -10;
				cyanRange = -8;
			}
			else if (level < 29) { //
				greenRange = -11;
				cyanRange = -8;
			}
			else if (level < 34) { // 
				greenRange = -12;
				cyanRange = -9;
			}
			else if (level < 37) { // 
				greenRange = -13;
				cyanRange = -10;
			}
			else if (level < 41) { // 37 - 40
				greenRange = -14;
				cyanRange = -11;
			}
			else if (level < 45) { // 41 - 44
				greenRange = -16;
				cyanRange = -12;
			}
			else if (level < 49) {  // 45 - 48
				greenRange = -17;
				cyanRange = -13;
			}
			else if (level < 53) { // 49 - 52
				greenRange = -18;
				cyanRange = -14;
			}
			else if (level < 55) { // 52 - 54
				greenRange = -19;
				cyanRange = -15;
			}
			else if (level < 57) { // 55 - 56
				greenRange = -20;
				cyanRange = -15;
			}
			else if (level < 61) { // 57 - 60
				greenRange = -21;
				cyanRange = -16;
			}
			else if (level == 61) { //61+
				greenRange = -19;
				cyanRange = -14;
			}
			else if (level == 62) {
				greenRange = -18;
				cyanRange = -13;
			}
			else if (level == 63) {
				greenRange = -17;
				cyanRange = -12;
			}
			else if (level < 66) { //64 - 65
				greenRange = -16;
				cyanRange = -11;
			}
			
			for (c = 0; c < level+greenRange; c++)  {
				conColors[c] = new SolidBrush(Color.Green);
			}

			for (; c < level+cyanRange; c++)  {
				conColors[c] = new SolidBrush(Color.Cyan);
			}

			for (; c < level; c++)  
				conColors[c] = new SolidBrush(Color.Blue);

			conColors[level] = new SolidBrush(Color.White);
			conColors[level+1] = new SolidBrush(Color.Yellow);
			conColors[level+2] = new SolidBrush(Color.Yellow);
			
		}


		private SolidBrush whiteBrush = new SolidBrush(Color.White);
		private Pen whitePen = new Pen(new SolidBrush(Color.White));
		private SolidBrush purpleBrush = new SolidBrush(Color.MediumPurple);
		private Pen ltbluePen = new Pen(new SolidBrush(Color.LightSteelBlue), 1);
		private SolidBrush greyBrush = new SolidBrush(Color.LightGray);
		private SolidBrush redBrush = new SolidBrush(Color.Red);
		private Pen ltgrayPen = new Pen(new SolidBrush(Color.LightGray));

		private void Form1_Paint(object sender, System.Windows.Forms.PaintEventArgs pe) {
			MapPane mp = (MapPane)Parent;
			Form1 f1 = (Form1)mp.Parent;
			if (f1.WindowState != FormWindowState.Minimized)  {
				// ************** GDI DRAWING **********************		
				Graphics sg = pe.Graphics;

				g.Clear(Settings.Instance.BackColor);

				if (longname != "")  {
					//for (int c = 0; c < numlines; c++)  {
					foreach (MapLine l in lines)  {
						for (int d = 1; d < l.numpoints; d++)  {
							//if (lines[c].color != null)
							g.DrawLine(l.color,l.points[d-1].x*(rawscale*scale)+(offsetx+offsetxg), l.points[d-1].y*(rawscale*scale)+(offsety+offsetyg), l.points[d].x*(rawscale*scale)+(offsetx+offsetxg), l.points[d].y*(rawscale*scale)+(offsety+offsetyg));
						}
					}

				//	for (int c = 0; c < numtexts; c++)  {
					foreach (MapText t in texts)  {
						//if (texts[c].color != null)
						g.DrawString(t.text, drawFont, t.color, t.x*(rawscale*scale)+(offsetx+offsetxg), t.y*(rawscale*scale)+(offsety+offsetyg));
					}
				}


				g.FillEllipse(redBrush, -playerinfo.Y*(rawscale*scale)+(offsetx+offsetxg)-2, -playerinfo.X*(rawscale*scale)+(offsety+offsetyg)-2, 5,5);	
				double realhead = playerinfo.Heading/512.0*360.0;
				float y1 = -(int)(Math.Cos(realhead*3.14159265358979323846/180.0)*(Settings.Instance.RangeCircle*rawscale*scale));
				float x1 = -(int)(Math.Sin(realhead*3.14159265358979323846/180.0)*(Settings.Instance.RangeCircle*rawscale*scale));
				g.DrawLine(whitePen, -playerinfo.Y*(rawscale*scale)+(offsetx+offsetxg), -playerinfo.X*(rawscale*scale)+(offsety+offsetyg), -playerinfo.Y*(rawscale*scale)+(offsetx+offsetxg)+x1, -playerinfo.X*(rawscale*scale)+(offsety+offsetyg)+2+y1);

				if (Settings.Instance.RangeCircle > 0)
					g.DrawEllipse(whitePen, -playerinfo.Y*(rawscale*scale)+(offsetx+offsetxg)-(Settings.Instance.RangeCircle*rawscale*scale), -playerinfo.X*(rawscale*scale)+(offsety+offsetyg)-(Settings.Instance.RangeCircle*rawscale*scale), (Settings.Instance.RangeCircle*rawscale*scale*2), (Settings.Instance.RangeCircle*rawscale*scale*2));

				lock(mobs)  {
					for (int c = 0; c < mobs.Count; c++)  {
						SPAWNINFO sp = (SPAWNINFO)mobs[c];
						if (selectedID == sp.SpawnID)  {
							g.DrawLine(whitePen, -playerinfo.Y*(rawscale*scale)+(offsetx+offsetxg), -playerinfo.X*(rawscale*scale)+(offsety+offsetyg), -sp.Y*(rawscale*scale)+(offsetx+offsetxg), -sp.X*(rawscale*scale)+(offsety+offsetyg));
						}

						if (sp.SpawnID != playerinfo.SpawnID && sp.flags == 0 && sp.Name != "")  {
							if (curTarget == sp.Name)   
								g.FillRectangle(whiteBrush, -sp.Y*(rawscale*scale)+(offsetx+offsetxg)-4, -sp.X*(rawscale*scale)+(offsety+offsetyg)-4, 9, 9);

							if (sp.Hide != 0 || sp.Race == 127)  {
								g.FillRectangle(greyBrush, -sp.Y*(rawscale*scale)+(offsetx+offsetxg)-2, -sp.X*(rawscale*scale)+(offsety+offsetyg)-2, 5,5);
							}
							else if (sp.Type == 0)  {
								if (conColors[sp.Level] != null)  {
									if (Settings.Instance.DontConPlayers)
										g.FillEllipse(purpleBrush, -sp.Y*(rawscale*scale)+(offsetx+offsetxg)-2, -sp.X*(rawscale*scale)+(offsety+offsetyg)-2, 5,5);
									else
										g.FillEllipse(conColors[sp.Level], -sp.Y*(rawscale*scale)+(offsetx+offsetxg)-2, -sp.X*(rawscale*scale)+(offsety+offsetyg)-2, 5,5);
								}
							}
							else if (sp.Type == 1 || sp.Type == 4)  {
								if (conColors[sp.Level] != null)
									g.FillEllipse(conColors[sp.Level], -sp.Y*(rawscale*scale)+(offsetx+offsetxg)-2, -sp.X*(rawscale*scale)+(offsety+offsetyg)-2, 5,5);
							}
							else if (sp.Type == 2 || sp.Type == 3)  {
								g.DrawEllipse(ltgrayPen, -sp.Y*(rawscale*scale)+(offsetx+offsetxg)-2, -sp.X*(rawscale*scale)+(offsety+offsetyg)-2, 5,5);
							}
						}
					}
				}				
				sg.DrawImageUnscaled(offScreenBmp, 0, 0, Width, Height);

			}			
		}
	}
}
